'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _setPrototypeOf = require('babel-runtime/core-js/object/set-prototype-of');

var _setPrototypeOf2 = _interopRequireDefault(_setPrototypeOf);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _testSetup = require('./test-setup');

var _libinit = require('../src/libinit');

var _index = require('../src/index');

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = (0, _create2.default)(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) _setPrototypeOf2.default ? (0, _setPrototypeOf2.default)(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } } /*
                                                                                                                                                           ******************************************************************************
                                                                                                                                                           Copyright (c) 2016 Particle Industries, Inc.  All rights reserved.
                                                                                                                                                          
                                                                                                                                                           This program is free software; you can redistribute it and/or
                                                                                                                                                           modify it under the terms of the GNU Lesser General Public
                                                                                                                                                           License as published by the Free Software Foundation, either
                                                                                                                                                           version 3 of the License, or (at your option) any later version.
                                                                                                                                                          
                                                                                                                                                           This program is distributed in the hope that it will be useful,
                                                                                                                                                           but WITHOUT ANY WARRANTY; without even the implied warranty of
                                                                                                                                                           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
                                                                                                                                                           Lesser General Public License for more details.
                                                                                                                                                          
                                                                                                                                                           You should have received a copy of the GNU Lesser General Public
                                                                                                                                                           License along with this program; if not, see <http://www.gnu.org/licenses/>.
                                                                                                                                                           ******************************************************************************
                                                                                                                                                           */

var fs = require('fs');
var path = require('path');
var helpers = require('yeoman-test');
var assert = require('yeoman-assert');
var fse = require('fs-extra');
var timekeeper = require('timekeeper');

// http://yeoman.io/generator/module-test_helpers.html

var EmptyBase = function EmptyBase() {
	_classCallCheck(this, EmptyBase);
};

var MockLibraryInitGenerator = function (_LibraryInitGenerator) {
	_inherits(MockLibraryInitGenerator, _LibraryInitGenerator);

	// eslint-disable-line new-cap
	function MockLibraryInitGenerator() {
		var _ref;

		_classCallCheck(this, MockLibraryInitGenerator);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _possibleConstructorReturn(this, (_ref = MockLibraryInitGenerator.__proto__ || (0, _getPrototypeOf2.default)(MockLibraryInitGenerator)).call.apply(_ref, [this].concat(args)));
	}

	return MockLibraryInitGenerator;
}((0, _libinit.LibraryInitGeneratorMixin)(EmptyBase));

describe('library initialize', function doit() {
	var testData = { name: 'nominative', version: '1.2.3', author: 'Borges <borges@example.com>' };

	before(function freezeTime() {
		timekeeper.freeze(Date.parse('2015-12-15'));
	});

	after(function releaseTime() {
		timekeeper.reset();
	});

	/**
  * Asserts that a generated (actual) file matches the expected file in the test fixture.
  * @param {string} expected  The name of the expected file. This is read from the `generator` folder at
  *  the same level as these tests.
  * @param {string} actual    The name of the actual file created. Assumes equal to 'expected' if not defined.
  * @returns {nada} nothing
  */
	function assertGeneratedContent(expected, actual) {
		if (actual === undefined) {
			actual = expected;
		}
		var expectedContent = fs.readFileSync(path.join(__dirname, './generator/', expected), 'utf8');
		assert.fileContent(actual, expectedContent);
	}

	/**
  * Validates that the library files are created that match the files in the test fixture.
  * @returns {undefined} nothing. nada. I exist only to pacify linting rules.
  */
	function validateOutput() {
		assertGeneratedContent('library.properties');
		assertGeneratedContent('README.md');
		assertGeneratedContent('LICENSE');
		assertGeneratedContent('src/nominative.cpp');
		assertGeneratedContent('src/nominative.h');
		assertGeneratedContent('examples/usage/usage.ino');
	}

	/**
  * Creates a LibraryInitGenerator and makes the source content equal to the
  * `src/init` directory. The generator is run in a temporary directory.
  * @param {string} dir       The directory under the source folder that contains the generator sources
  * @param {function} cb      A callback that is passed the created generator.
  * @returns {Promise}   To run the generator.
  */
	function generator(dir, cb) {
		var result = helpers.run((0, _libinit.buildLibraryInitGeneratorClass)());
		if (dir) {
			result = result.inTmpDir(function (tmpdir) {
				// `tmpdir` is the path to the new temporary directory
				fse.copySync(path.join(_index.appRoot, 'src', dir), tmpdir);
			});
		}
		if (cb) {
			result = cb(result);
		}
		return result.toPromise();
	}

	describe('generator', function () {
		it('interpolates all template files', function doit() {
			this.timeout(30000);
			return generator('init', function (result) {
				return result.withOptions(testData); // Mock options passed in
			}).then(validateOutput);
		});

		it('should prompt for all properties if not provided', function () {
			return generator('init', function (result) {
				return result.withPrompts(testData); // Mock options passed in
			}).then(validateOutput);
		});

		it('sets the output directory from the "dir" option', function () {
			var sut = new MockLibraryInitGenerator();
			sut.options = { dir: 'abcd' };
			sut.destinationRoot = _testSetup.sinon.stub();
			sut._setOutputDir();
			(0, _testSetup.expect)(sut.destinationRoot).to.have.been.calledWith('abcd');
		});

		it('does not set the output directory when the dir option is not present', function () {
			var sut = new MockLibraryInitGenerator();
			sut.options = {};
			sut.destinationRoot = _testSetup.sinon.stub();
			sut._setOutputDir();
			(0, _testSetup.expect)(sut.destinationRoot).to.have.not.been.called;
		});

		it('sets the name_code option to the code-safe name with the first letter lowercased', function () {
			var sut = new MockLibraryInitGenerator();
			sut.options = {};
			sut._handlePrompts({ name: 'SparkLib++' });
			(0, _testSetup.expect)(sut.options).to.have.property('name_code').equal('sparkLib');
		});

		it('sets the Name_code option to the code-safe name with first letter capitalized', function () {
			var sut = new MockLibraryInitGenerator();
			sut.options = {};
			sut._handlePrompts({ name: 'my-lib++' });
			(0, _testSetup.expect)(sut.options).to.have.property('Name_code').equal('Mylib');
		});

		it('does not set the Name_code option when name is not present', function () {
			var sut = new MockLibraryInitGenerator();
			sut.options = {};
			sut._handlePrompts({ name2: 'abcd' });
			(0, _testSetup.expect)(sut.options).to.not.have.property('Name_code');
		});

		describe('validation', function () {
			var nameError = 'name: must only contain letters, numbers, dashes, underscores and plus signs';
			var versionError = 'version: must be formatted like 1.0.0';
			it('validates the name', function () {
				var sut = new MockLibraryInitGenerator();
				sut.options = {};
				(0, _testSetup.expect)(function () {
					return sut._handlePrompts({ name: 'ab/cd' });
				}).to.throw(nameError);
			});

			it('validates the version', function () {
				var sut = new MockLibraryInitGenerator();
				sut.options = {};
				(0, _testSetup.expect)(function () {
					return sut._handlePrompts({ version: 'ab/cd' });
				}).to.throw(versionError);
			});

			it('validates the author, which is freeform', function () {
				var sut = new MockLibraryInitGenerator();
				sut.options = {};
				(0, _testSetup.expect)(function () {
					return sut._handlePrompts({ author: 'ab/cd' });
				}).to.not.throw();
			});

			it('validates the initial name value', function () {
				var sut = new MockLibraryInitGenerator();
				sut.options = { name: '//' };
				(0, _testSetup.expect)(function () {
					return sut._checkFields();
				}).to.throw(nameError);
			});

			it('validates the initial version value', function () {
				var sut = new MockLibraryInitGenerator();
				sut.options = { version: '//' };
				(0, _testSetup.expect)(function () {
					return sut._checkFields();
				}).to.throw(versionError);
			});

			it('validates the initial version value when set as a number', function () {
				var sut = new MockLibraryInitGenerator();
				sut.options = { version: 123 };
				(0, _testSetup.expect)(function () {
					return sut._checkFields();
				}).to.throw(versionError);
			});

			it('validates the prompts', function () {
				var sut = new MockLibraryInitGenerator();
				sut.options = {};
				var prompts = sut._allPrompts();
				(0, _testSetup.expect)(prompts).has.property('length').equal(3);

				(0, _testSetup.expect)(prompts[0].validate('ab/cd')).to.equal(nameError + '.');
				(0, _testSetup.expect)(prompts[1].validate('ab/cd')).to.equal(versionError);
				(0, _testSetup.expect)(prompts[2].validate('ab/cd')).to.be.true;
			});
		});

		it('prompting delegates to the _prompt method', function () {
			var sut = new MockLibraryInitGenerator();
			(0, _testSetup.expect)(sut.prompting).to.have.property('prompt').equal(sut._prompt);
		});

		it('the _prompt method configures and fetches options', function () {
			var sut = new MockLibraryInitGenerator();
			// given
			sut._setYear = _testSetup.sinon.stub();
			sut._setOutputDir = _testSetup.sinon.stub();
			sut._allPrompts = _testSetup.sinon.stub().returns('abcd');
			sut._handlePrompts = _testSetup.sinon.stub().returns('handled');
			sut.prompt = _testSetup.sinon.stub().returns(_promise2.default.resolve({ name: '123' }));
			// when
			return sut._prompt().then(function (result) {
				(0, _testSetup.expect)(result).to.be.equal('handled');
				(0, _testSetup.expect)(sut._handlePrompts).to.have.been.calledWith({ name: '123' });
				(0, _testSetup.expect)(sut._allPrompts).to.have.been.calledOnce;
				(0, _testSetup.expect)(sut._setOutputDir).to.have.been.calledOnce;
			});
		});
	});
});
//# sourceMappingURL=libinit.spec.js.map